/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.action;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.utils.EMFResourceUtils;

/**
 * Methods for simplifying copy/paste.
 * 
 * @author hummel
 * @author hoelzl
 * 
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
final class CopyPasteUtils {
	/** Copies the given selected objects to the clipboard. */
	public static void copyToClipboard(Collection<EObject> selection)
			throws IOException {
		ByteArrayOutputStream bytes = new ByteArrayOutputStream();
		Resource resource = new ResourceSetImpl().createResource(URI
				.createPlatformResourceURI("file://does/not/exist", true));
		for (EObject sel : selection) {
			resource.getContents().add(EcoreUtil.copy(sel));
		}
		resource.save(bytes, EMFResourceUtils.buildOptionsMap());
		String text = bytes.toString();

		Clipboard clipboard = new Clipboard(Display.getDefault());
		clipboard.setContents(new Object[] { text },
				new Transfer[] { TextTransfer.getInstance() }, DND.CLIPBOARD
						| DND.SELECTION_CLIPBOARD);
		clipboard.dispose();
	}

	/**
	 * Attempts to paste the contents of the clipboard into the given target
	 * object.
	 */
	public static void pasteFromClipboard(EObject target) {
		Clipboard clipboard = new Clipboard(Display.getDefault());
		String text = (String) clipboard
				.getContents(TextTransfer.getInstance());
		clipboard.dispose();

		if (text == null) {
			return;
		}

		List<EObject> contents = new ArrayList<EObject>();
		try {
			ByteArrayInputStream in = new ByteArrayInputStream(text.getBytes());
			Resource resource = new ResourceSetImpl().createResource(URI
					.createPlatformResourceURI("file://does/not/exist", true));
			resource.load(in, EMFResourceUtils.buildOptionsMap());
			contents.addAll(resource.getContents());
		} catch (Exception e) {
			MessageDialog.openError(Display.getDefault().getActiveShell(),
					"Paste",
					"Clipboard contents not readable: " + e.getMessage());
			return;
		}

		for (EObject insertObj : contents) {
			if (!IElementCompositorService.INSTANCE
					.canCompose(target, insertObj, null)) {
				return;
			}
		}

		for (EObject insertObj : contents) {
			IElementCompositorService.INSTANCE.compose(target, insertObj, null);
			// LibrarySupport.activateAllElements(insertObj);
		}
	}
}
