/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.action;

import java.io.IOException;
import java.util.Collection;

import org.eclipse.emf.common.command.AbstractCommand;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.base.ui.ToolingBaseUIActivator;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * Action for cutting the current selection.
 * 
 * @author hummel
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class CutModelElementsAction extends ModelElementsActionBase {

	/** Constructor. */
	public CutModelElementsAction() {
		super("Cut", ActionFactory.CUT.getId(), PlatformUI.getWorkbench()
				.getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_CUT), PlatformUI
				.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_CUT_DISABLED), true);
	}

	/** {@inheritDoc} */
	@Override
	protected Command createCommand(final Collection<EObject> selection) {
		return new AbstractCommand("Cut") {

			@Override
			public void execute() {
				try {
					CopyPasteUtils.copyToClipboard(selection);
					for (EObject o : selection) {
						IElementCompositorService.INSTANCE.decompose(o);
					}
				} catch (IOException e) {
					LoggingUtils.error(ToolingBaseUIActivator.getDefault(),
							"Had problems during cut operation", e);
				}
			}

			@Override
			public void redo() {
				// not needed as our transactional command stack
				// handles this
			}

			@Override
			public boolean canExecute() {
				for (EObject o : selection) {
					if (!IElementCompositorService.INSTANCE.canDecompose(o)) {
						return false;
					}
				}
				return true;
			}
		};
	}
}
