/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.command;

import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.OffsetOrientation;
import org.fortiss.tooling.base.ui.editpart.FreeConnectorEditPartBase;
import org.fortiss.tooling.base.ui.editpart.StickyConnectorEditPartBase;
import org.fortiss.tooling.base.ui.layout.constants.LayoutKeyConstants;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;

/**
 * A command to move connectors. The operation is performed differently for
 * sticky and free connector edit parts.
 * 
 * @author wandinger
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: E806F6189F17B9B9E2A126FB5432A81A
 */
public class MoveConnectorCommand extends ChangeBoundsRequestCommandBase {

	/** Constructor. */
	public MoveConnectorCommand(ChangeBoundsRequest request) {
		super(request);
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void execute() {
		super.execute();
		for (EditPart editpart : (List<EditPart>) getRequest().getEditParts()) {
			performMove(editpart, getRequest());
		}
	}

	/** Performs the move of an edit part. */
	protected void performMove(EditPart editpart, ChangeBoundsRequest request) {
		if (editpart instanceof StickyConnectorEditPartBase) {
			// The orientation was already calculated for the sourceFeedback
			OffsetOrientation offsetOrientation = (OffsetOrientation) request
					.getExtendedData().get(
							LayoutKeyConstants.CONNECTOR_OFFSET_ORIENTATION);
			LayoutDataUtils.setConnectorOffsetOrientation(
					(ILayoutedModelElement) editpart.getModel(),
					offsetOrientation);

		} else if (editpart instanceof FreeConnectorEditPartBase) {
			moveNode((ILayoutedModelElement) editpart.getModel());
		}
	}
}
