/*--------------------------------------------------------------------------+
 $Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.dnd;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.dnd.DND;
import org.fortiss.tooling.base.ui.editor.gef.GraphicalViewerEditorBase;

/**
 * The drop listener used for {@link GraphicalViewerEditorBase}.
 * 
 * @author hummel
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 8254A8D604E8FA36921DB38A8F8C717B
 */
public class DropTargetListener extends AbstractTransferDropTargetListener {

	/** CreateFactory for this drop target listener */
	private final GefCreationFactory creationFactory = new GefCreationFactory();

	/** Constructor. */
	public DropTargetListener(EditPartViewer viewer) {
		super(viewer, LocalSelectionTransfer.getTransfer());

		setEnablementDeterminedByCommand(true);
	}

	/** {@inheritDoc} */
	@Override
	protected Request createTargetRequest() {
		CreateRequest request = new CreateRequest();
		request.setFactory(creationFactory);
		return request;
	}

	/** {@inheritDoc} */
	@Override
	public void handleDragOver() {
		getCurrentEvent().detail = DND.DROP_COPY;
		creationFactory.setTemplate(getDropObject());
		super.handleDragOver();
	}

	/** {@inheritDoc} */
	@Override
	public void handleDrop() {
		creationFactory.setTemplate(getDropObject());
		super.handleDrop();
	}

	/** {@inheritDoc} */
	@Override
	protected void updateTargetRequest() {
		creationFactory.setTemplate(getDropObject());
		((CreateRequest) getTargetRequest()).setLocation(getDropLocation());
	}

	/** Returns the objects currently being dropped. */
	private Object getDropObject() {
		ISelection selection = LocalSelectionTransfer.getTransfer()
				.getSelection();
		Object firstElement = ((IStructuredSelection) selection)
				.getFirstElement();
		return firstElement;
	}

	/** The {@link CreationFactory} used for drag and drop operations. */
	private static class GefCreationFactory implements CreationFactory {
		/** Currently used template (UnitDescriptor or PortTemplate) */
		private Object template;

		/** {@inheritDoc} */
		@Override
		public Object getNewObject() {
			return template;
		}

		/** Sets all needed information to create the object. */
		/* package */void setTemplate(Object template) {
			this.template = template;
		}

		/** {@inheritDoc} */
		@Override
		public Object getObjectType() {
			return null;
		}
	}
}
