/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.draw2d.geometry.Rectangle;
import org.fortiss.tooling.base.model.base.EntryConnectorBase;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.layout.constants.DefaultLayoutConstants;
import org.fortiss.tooling.base.ui.layout.util.RectangleLayoutUtils;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * Abstract implementation for sticky connector edit parts. Usually, the
 * connector sticks to some parent edit part, e.g., a connector moving only on
 * the border of its parent element when the parent is viewed externally.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public abstract class StickyConnectorEditPartBase<T extends ILayoutedModelElement & IConnector & INamedCommentedElement>
		extends ConnectorEditPartBase<T> {

	/** Constructor. */
	protected StickyConnectorEditPartBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determineBaseFigureBounds() {
		if (modelElement.eContainer() instanceof ILayoutedModelElement) {
			return RectangleLayoutUtils.getAbsoluteConnectorBounds(
					modelElement,
					(ILayoutedModelElement) modelElement.eContainer());
		}
		return new Rectangle(0, 0,
				DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE,
				DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE);
	}

	/** {@inheritDoc} */
	@Override
	protected boolean isTarget() {
		return modelElement instanceof EntryConnectorBase;
	}
}
