/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.BendpointEditPolicy;
import org.eclipse.gef.requests.BendpointRequest;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.command.CreateBendpointCommand;
import org.fortiss.tooling.base.ui.command.DeleteBendpointCommand;
import org.fortiss.tooling.base.ui.command.MoveBendpointCommand;
import org.fortiss.tooling.base.ui.editpart.ConnectionEditPartBase;
import org.fortiss.tooling.base.ui.layout.IPointLayouter;
import org.fortiss.tooling.base.ui.layout.util.SnapToGridUtils;

/**
 * An edit policy for bend points of connections.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: E77983D22C4E49D977F8A4BC13E34D85
 */
public class ConnectionBendpointEditPolicy<T extends ILayoutedModelElement & IConnection>
		extends BendpointEditPolicy implements IPointLayouter {
	/** The channel of the bend point. */
	private final ConnectionEditPartBase<T> part;

	/** Constructor. */
	public ConnectionBendpointEditPolicy(ConnectionEditPartBase<T> part) {
		this.part = part;
	}

	/** {@inheritDoc} */
	@Override
	protected Command getCreateBendpointCommand(BendpointRequest request) {
		Point location = request.getLocation();
		part.getFigure().translateToRelative(location);
		return new CreateBendpointCommand(part.getModelElement(),
				request.getIndex(), location, this);
	}

	/** {@inheritDoc} */
	@Override
	protected Command getDeleteBendpointCommand(BendpointRequest request) {
		return new DeleteBendpointCommand(part.getModelElement(),
				request.getIndex());
	}

	/** {@inheritDoc} */
	@Override
	protected Command getMoveBendpointCommand(BendpointRequest request) {
		Point location = request.getLocation();
		part.getFigure().translateToRelative(location);

		return new MoveBendpointCommand(part.getModelElement(),
				request.getIndex(), location, this);
	}

	/** {@inheritDoc} */
	@Override
	protected void showCreateBendpointFeedback(BendpointRequest request) {
		doLayout(request.getLocation());
		super.showCreateBendpointFeedback(request);
	}

	/** {@inheritDoc} */
	@Override
	protected void showMoveBendpointFeedback(BendpointRequest request) {
		doLayout(request.getLocation());
		super.showMoveBendpointFeedback(request);
	}

	/** {@inheritDoc} */
	@Override
	public void doLayout(Point p) {
		p.x = SnapToGridUtils.snapToGrid(p.x);
		p.y = SnapToGridUtils.snapToGrid(p.y);
	}
}
