/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.gef.Request;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.ui.layout.util.RectangleLayoutUtils;

/**
 * EditPolicy for child resizing without a fixed aspect ratio.
 * <p>
 * This EditPolicy is responsible for resizing child parts. A minimal size is
 * ensured. Resizing is restricted to the east, south-east and south handles.
 * All other handles are used for moves only.
 * </p>
 * 
 * @author wandinger
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 8F9361223B2D04A638954A22CE574C86
 */
public class FreeResizableEditPolicy extends ResizableEditPolicy {

	/** Constructor. */
	public FreeResizableEditPolicy() {
		// enable resizing on east, south-east and south handles.
		setResizeDirections(PositionConstants.SOUTH_EAST);
	}

	/** {@inheritDoc} */
	@Override
	public void showSourceFeedback(Request request) {
		if (REQ_RESIZE.equals(request.getType())) {
			Dimension oldSizeDelta = ((ChangeBoundsRequest) request)
					.getSizeDelta();
			Dimension oldSize = new Dimension(getHostFigure().getBounds().width
					+ oldSizeDelta.width, getHostFigure().getBounds().height
					+ oldSizeDelta.height);
			Dimension newSize = RectangleLayoutUtils
					.clampRectangularShapeSize(oldSize);
			Dimension newSizeDelta = new Dimension(newSize.width
					- getHostFigure().getBounds().width, newSize.height
					- getHostFigure().getBounds().height);

			((ChangeBoundsRequest) request).setSizeDelta(newSizeDelta);
		}
		super.showSourceFeedback(request);
	}
}
