/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.requests.CreateRequest;
import org.fortiss.tooling.base.ui.dnd.DropContext;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.util.DragAndDropUtils;

/**
 * Base class for layout edit policies relaying create commands to
 * {@link IElementCompositorService}.
 * 
 * @author hummel
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 6D1500C8A3A09024E500FF7DEBE8776D
 */
public abstract class LayoutEditPolicyBase extends LayoutEditPolicy {

	/** {@inheritDoc} */
	@Override
	protected Command getCreateCommand(CreateRequest request) {
		/*
		 * when creating object user is not allowed to drag otherwise the object
		 * would be set to elsewhere.
		 * 
		 * Each command gets its own location field because the
		 * request.getLocation() is changed before executing the command
		 */

		if (request.getSize() != null) {
			return null;
		}
		if (!(getHost().getModel() instanceof EObject)) {
			return null;
		}

		Point location = request.getLocation();
		((GraphicalEditPart) getHost()).getFigure().translateToRelative(
				location);

		final EObject model = (EObject) getHost().getModel();
		final EObject dropped = DragAndDropUtils.extractDroppedEObject(request
				.getNewObject());
		final IElementCompositionContext context = new DropContext(getHost(),
				location);
		return new Command() {
			@Override
			public boolean canExecute() {
				return IElementCompositorService.INSTANCE.canCompose(model,
						dropped, context);
			}

			@Override
			public void execute() {
				IElementCompositorService.INSTANCE.compose(model, dropped,
						context);
			}
		};
	}
}
