/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout.util;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.LayoutFactory;
import org.fortiss.tooling.base.model.layout.Point;

/**
 * TODO: What is the difference between {@link PointUtil} and {@link PointsUtil}
 * ?
 * 
 * Utility class for handling {@link Point} layout data.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: ECCAD8A00B043640F19A975F3B45B36B
 */
/* package */final class PointUtil {

	/** Creates a new {@link Point}. */
	public static Point createPoint(int x, int y) {
		Point p = LayoutFactory.eINSTANCE.createPoint();
		p.setX(x);
		p.setY(y);
		return p;
	}

	/** Gets the position stored at the given key. */
	public static Point getPosition(ILayoutedModelElement layouted, String key) {
		Point point = getPoint(layouted, key);

		if (point == null) {
			int index = 0;
			EObject container = layouted.eContainer();
			if (container != null) {
				index = 1 + container.eContents().indexOf(layouted);
			}
			Point p = createPoint(index * 20, index * 20);
			p.setKey(key);
			return p;
		}
		return EcoreUtil.copy(point);
	}

	/** Returns a {@link Point} layout data object with the given key or null. */
	private static Point getPoint(ILayoutedModelElement lobject, String key) {
		for (ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) && ld instanceof Point) {
				return (Point) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Point} layout data. */
	public static void setPoint(ILayoutedModelElement lobject, String key,
			int x, int y) {
		Point old = getPoint(lobject, key);
		if (old != null) {
			lobject.getLayoutDataList().remove(old);
		}

		Point p = createPoint(x, y);
		p.setKey(key);
		lobject.getLayoutDataList().add(p);
	}
}
