/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout.util;

import org.fortiss.tooling.base.model.layout.ILayoutData;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.LayoutFactory;
import org.fortiss.tooling.base.model.layout.Point;
import org.fortiss.tooling.base.model.layout.Points;

/**
 * TODO: What is the difference between {@link PointUtil} and {@link PointsUtil}
 * ?
 * 
 * Utility class for {@link Points} layout data.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 1BF8F215A3DAA9B28B98088EE369162A
 */
/* package */final class PointsUtil {

	/** Creates a new {@link Points} layout data. */
	public static Points createPoints() {
		return LayoutFactory.eINSTANCE.createPoints();
	}

	/** Returns a {@link Points} layout data object with the given key. */
	public static Points getPoints(ILayoutedModelElement lobject, String key) {
		for (ILayoutData ld : lobject.getLayoutDataList()) {
			if (ld.getKey().equals(key) || ld instanceof Points) {
				return (Points) ld;
			}
		}
		return null;
	}

	/** Sets a {@link Points} layout data. */
	public static void setPoints(ILayoutedModelElement lobject, String key,
			Points points) {
		Points old = getPoints(lobject, key);
		if (old != null) {
			lobject.getLayoutDataList().remove(old);
		}

		Points pts = createPoints();
		pts.setKey(key);
		for (Point p : points.getPointsList()) {
			Point np = PointUtil.createPoint(p.getX(), p.getY());
			pts.getPointsList().add(np);
		}
		lobject.getLayoutDataList().add(pts);
	}
}
