/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import java.util.EventObject;

import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.listener.IPersistencyServiceListener;
import org.fortiss.tooling.kernel.ui.internal.actions.DeleteAction;
import org.fortiss.tooling.kernel.ui.internal.actions.RedoAction;
import org.fortiss.tooling.kernel.ui.internal.actions.UndoAction;
import org.fortiss.tooling.kernel.ui.service.IActionService;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * This class implements the {@link IActionService} interface.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: AA768AB233B1896A83B8C734A6C7EB3C
 */
public class ActionService implements IActionService,
		IPersistencyServiceListener, CommandStackListener {

	/** The menu section ID of the global default action group. */
	public static final String GLOBAL_DEFAULT_MENU_SECTION_ID = "globalDefaults";

	/** The global delete action. */
	public final DeleteAction globalDeleteAction;

	/** The global undo action. */
	public final UndoAction globalUndoAction;

	/** The global redo action. */
	public final RedoAction globalRedoAction;

	/** Constructor. */
	public ActionService() {
		globalDeleteAction = new DeleteAction();
		globalUndoAction = new UndoAction();
		globalRedoAction = new RedoAction();

		for (ITopLevelElement context : IPersistencyService.INSTANCE
				.getTopLevelElements()) {
			context.addCommandStackListener(this);
		}
		IPersistencyService.INSTANCE.addTopLevelElementListener(this);
	}

	/** {@inheritDoc} */
	@Override
	public void addGlobalDefaultActionSectionToMenu(IMenuManager menuManager) {
		menuManager.appendToGroup(GLOBAL_DEFAULT_MENU_SECTION_ID,
				globalDeleteAction);
		menuManager.appendToGroup(GLOBAL_DEFAULT_MENU_SECTION_ID,
				globalUndoAction);
		menuManager.appendToGroup(GLOBAL_DEFAULT_MENU_SECTION_ID,
				globalRedoAction);
	}

	/** {@inheritDoc} */
	@Override
	public void registerGlobalActions(IActionBars actionBars) {
		globalUndoAction.setActionDefinitionId(ActionFactory.UNDO
				.getCommandId());
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
				globalUndoAction);

		globalRedoAction.setActionDefinitionId(ActionFactory.REDO
				.getCommandId());
		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(),
				globalRedoAction);

		actionBars.updateActionBars();
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		EObject target = EObjectSelectionUtils
				.getCurrentSelectionFirstElement();
		globalDeleteAction.refresh(target);
		globalUndoAction.refresh(target);
		globalRedoAction.refresh(target);
	}

	/** {@inheritDoc} */
	@Override
	public void commandStackChanged(EventObject event) {
		refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementAdded(ITopLevelElement elementContext) {
		elementContext.addCommandStackListener(this);
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementRemoved(ITopLevelElement elementContext) {
		elementContext.removeCommandStackListener(this);
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementChanged(ITopLevelElement element) {
		// ignore these changes
	}
}
