/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.ui.extension.IModelEditorBinding;
import org.fortiss.tooling.kernel.ui.internal.editor.BindingEditor;
import org.fortiss.tooling.kernel.ui.internal.editor.ModelElementEditorInput;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * This class implements the {@link IModelEditorBindingService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: 5544DDDB18F4389B85B35A1CAA58FD17
 */
public class ModelEditorBindingService extends
		EObjectAwareServiceBase<IModelEditorBinding<EObject>> implements
		IModelEditorBindingService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.ui.modelEditorBinding";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelEditorBinding";

	/** The handler class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "binding";

	/** Constructor. */
	public ModelEditorBindingService() {
		super();
		// sort editor bindings by priority.
		EditorBindingComparator comparator = new EditorBindingComparator();
		for (Class<?> clazz : handlerMap.keySet()) {
			Collections.sort(handlerMap.get(clazz), comparator);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void openInEditor(EObject element) {
		if (getBindings(element).isEmpty()) {
			return;
		}
		try {
			PlatformUI
					.getWorkbench()
					.getActiveWorkbenchWindow()
					.getActivePage()
					.openEditor(new ModelElementEditorInput(element),
							BindingEditor.ID);
		} catch (final PartInitException e) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Could not open editor with ID " + BindingEditor.ID, e);
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IModelEditorBinding<EObject>> getBindings(EObject element) {
		List<IModelEditorBinding<EObject>> bindings = getRegisteredHandlers(element
				.getClass());
		if (bindings == null) {
			bindings = CollectionUtils.emptyList();
		}
		return bindings;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}

	/** {@link Comparator} for {@link IModelEditorBinding}s. */
	private static class EditorBindingComparator implements
			Comparator<IModelEditorBinding<EObject>> {

		/** {@inheritDoc} */
		@Override
		public int compare(IModelEditorBinding<EObject> eb0,
				IModelEditorBinding<EObject> eb1) {
			return eb1.getPriority() - eb0.getPriority();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void closeEditors(EObject rootElement) {
		// TODO (FH): implement

		// for (final IEditorPart editor : editors) {
		// editor.getSite().getShell().getDisplay().asyncExec(new Runnable() {
		// @Override
		// public void run() {
		// editor.getSite().getPage().closeEditor(editor, false);
		// }
		// });
		// }
	}
}
