/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;

/**
 * The {@link DeleteAction} uses the {@link IElementCompositorService} and the
 * {@link IConnectionCompositorService} to determine the enabled state of this
 * action.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: C34C3D95AE0D485EC73074D1CC36029D
 */
public class DeleteAction extends EObjectActionBase {

	/** Flag for using element or connection compositor. */
	private boolean useElementCompositor;

	/** Constructor. */
	public DeleteAction() {
		super("Delete", PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		setId(ActionFactory.DELETE.getId());
	}

	/** Refreshes the action's target and enabled state. */
	public void refresh(EObject element) {
		setTarget(element);
		if (element == null) {
			setEnabled(false);
			return;
		}

		boolean enabled = false;
		if (!IPersistencyService.INSTANCE.isTopLevelElement(element)) {
			if (IElementCompositorService.INSTANCE.canDecompose(element)) {
				enabled = true;
				useElementCompositor = true;
			} else if (IConnectionCompositorService.INSTANCE
					.canDisconnect(element)) {
				enabled = true;
				useElementCompositor = false;
			}
		}
		setEnabled(enabled);
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		final EObject selectedObject = getTarget();
		if (selectedObject == null) {
			return;
		}
		ICommandStackService.INSTANCE.runAsCommand(selectedObject,
				new Runnable() {
					@Override
					public void run() {
						if (useElementCompositor) {
							IElementCompositorService.INSTANCE
									.decompose(selectedObject);
						} else {
							IConnectionCompositorService.INSTANCE
									.disconnect(selectedObject);
						}
					}
				});
	}
}