/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;

/**
 * The {@link RedoAction} uses the {@link ICommandStackService} to determine the
 * enabled state of this action.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: FE20124DCB7DAF4D7AAAC0809FA5B5D2
 */
public class RedoAction extends EObjectActionBase {

	/** Constructor. */
	public RedoAction() {
		super("Redo", PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_REDO));
		setId(ActionFactory.REDO.getId());
	}

	/** Refreshes the action's target and enabled state. */
	public void refresh(EObject element) {
		setTarget(element);
		if (element == null) {
			setEnabled(false);
			return;
		}

		setEnabled(ICommandStackService.INSTANCE.canRedo(element));
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		if (getTarget() != null) {
			ICommandStackService.INSTANCE.redo(getTarget());
		}
	}
}
