/*-----------------------------------------------------------------------+
 | org.fortiss.tooling.kernel.ui.internal.views
 |                                                                       |
   $Id$            
 |                                                                       |
 | Copyright (c)  2004-2008 Technische Universitaet Muenchen             |
 |                                                                       |
 | Technische Universitaet Muenchen               #########  ##########  |
 | Institut fuer Informatik - Lehrstuhl IV           ##  ##  ##  ##  ##  |
 | Prof. Dr. Manfred Broy                            ##  ##  ##  ##  ##  |
 | Boltzmannstr. 3                                   ##  ##  ##  ##  ##  |
 | 85748 Garching bei Muenchen                       ##  ##  ##  ##  ##  |
 | Germany                                           ##  ######  ##  ##  |
 +-----------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.views;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPrototypeService;
import org.fortiss.tooling.kernel.ui.extension.IContextMenuContributor;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.service.IContextMenuService;
import org.fortiss.tooling.kernel.ui.service.IModelEditorBindingService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;
import org.fortiss.tooling.kernel.ui.service.INavigatorService;

/**
 * TODO: why is this class in the "views" package?
 * 
 * The contributor for the dynamic new menu based on composition.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: DE090EFA7BC79BE370B8C43D8DEADBF7
 */
public class NewMenu implements IContextMenuContributor {

	/** List of available actions. */
	private final List<AddPrototypeAction> actions = new ArrayList<AddPrototypeAction>();

	/** Constructor. */
	public NewMenu() {
		for (IPrototypeProvider provider : IPrototypeService.INSTANCE
				.getPrototypeProviders()) {
			for (Prototype prototype : provider.getPrototypes()) {
				actions.add(new AddPrototypeAction(prototype));
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public List<IContributionItem> getContributedItems(EObject selectedObject,
			ContextMenuContextProvider contextProvider) {
		if (!INavigatorService.INSTANCE.isNavigatorView(contextProvider)
				|| selectedObject == null) {
			return Collections.emptyList();
		}

		List<IContributionItem> contributionItems = new ArrayList<IContributionItem>();
		for (AddPrototypeAction apa : actions) {
			if (apa.prepare(selectedObject)) {
				contributionItems.add(new ActionContributionItem(apa));
			}
		}
		return contributionItems;
	}

	/** {@inheritDoc} */
	@Override
	public String getMenuSectionID() {
		return IContextMenuService.TOP_MOST_MENU_SECTION_ID;
	}

	/** Action for creating a new prototype. */
	private static class AddPrototypeAction extends Action {

		/** The prototype. */
		private final Prototype prototype;

		/** The container. */
		private EObject container;

		/** Constructor. */
		public AddPrototypeAction(Prototype prototype) {
			super(prototype.getName(), IModelElementHandlerService.INSTANCE
					.getModelElementHandler(prototype.getPrototype())
					.getIconImageDescriptor());
			this.prototype = prototype;
		}

		/** {@inheritDoc} */
		@Override
		public void run() {
			ICommandStackService.INSTANCE.runAsCommand(container,
					new Runnable() {

						@Override
						public void run() {
							EObject newObject = prototype.getPrototypeCopy();
							if (IElementCompositorService.INSTANCE.compose(
									container, newObject, null)) {
								IModelEditorBindingService.INSTANCE
										.openInEditor(newObject);
							}
						}
					});
		}

		/**
		 * Stores the given container and returns whether the underlying
		 * prototype can be added to it.
		 */
		public boolean prepare(EObject container) {
			this.container = container;
			return IElementCompositorService.INSTANCE.canCompose(container,
					prototype.getPrototype(), null);
		}

	}
}
