/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.service;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IConstraintCheckerService;
import org.fortiss.tooling.kernel.ui.internal.MarkerService;

/**
 * The marker service manages model element specific markers. It uses the
 * {@link IConstraintCheckerService} to check constraints on the model and
 * create the UI feedback from the results.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 579B2321FC685A2D551FAFF13A0FB3A6
 */
public interface IMarkerService {

	/** Returns the singleton instance of the service. */
	public static final IMarkerService INSTANCE = new MarkerService();

	/** Returns the constraint violations for the given element. */
	Collection<IConstraintViolation<? extends EObject>> getViolations(
			EObject element);

	/** Returns the highest severity for the given element. */
	ESeverity getHighestViolationSeverity(EObject element);

	/**
	 * Refreshes the markers of the given top-level element (and its children
	 * recursively).
	 */
	void refreshMarkers(ITopLevelElement element);
}
