/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import org.conqat.ide.commons.ui.databinding.JFaceObservables;
import org.eclipse.core.databinding.Binding;
import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.UpdateValueStrategy;
import org.eclipse.core.databinding.conversion.IConverter;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.validation.IValidator;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

/**
 * Utility methods for data binding support.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 1AA4C5FB4C31B3AEDFC4627BFCA48F02
 */
public final class DataBindingUtils {

	/** Decoration key for edit feedback. */
	public static final String DECORATION_KEY = ControlDecoration.class
			.getName();

	/**
	 * Performs a complex binding of a text control to a model element. The
	 * validation is performed on modification (i.e. always), while model
	 * updates are only performed on focus out (if validation works). There is
	 * no validation support from model to text, as the model is considered to
	 * be always consistent. If the control has a {@link ControlDecoration}
	 * stored under the {@link #DECORATION_KEY} key, this is used to visualize
	 * the validation result.
	 */
	public static Binding[] performComplexTextBinding(DataBindingContext dbc,
			Control control, IObservableValue modelValue,
			IConverter modelToTextConverter, IConverter textToModelConverter,
			IValidator textValidator, IValidator textPostConvertValidator) {

		Binding[] bindings = new Binding[3];

		// normal model to control
		UpdateValueStrategy modelToTextStrategy = new UpdateValueStrategy(true,
				UpdateValueStrategy.POLICY_UPDATE)
				.setConverter(modelToTextConverter);

		// use POLICY_UPDATE to actually perform the update
		UpdateValueStrategy textToModelStrategyWithUpdate = new UpdateValueStrategy(
				true, UpdateValueStrategy.POLICY_UPDATE)
				.setConverter(textToModelConverter)
				.setAfterGetValidator(textValidator)
				.setAfterConvertValidator(textPostConvertValidator);

		// add the "normal" binding to actually update the model
		bindings[0] = dbc.bindValue(
				SWTObservables.observeText(control, SWT.FocusOut), modelValue,
				textToModelStrategyWithUpdate, modelToTextStrategy);

		// here it is important to only use POLICY_CONVERT, to not write through
		// to the model
		UpdateValueStrategy textToModelStrategyNoUpdate = new UpdateValueStrategy(
				true, UpdateValueStrategy.POLICY_CONVERT)
				.setConverter(textToModelConverter)
				.setAfterGetValidator(textValidator)
				.setAfterConvertValidator(textPostConvertValidator);

		// perform a binding which only serves for validation purposes
		bindings[1] = dbc.bindValue(SWTObservables.observeText(control,
				SWT.Modify), modelValue, textToModelStrategyNoUpdate,
				new UpdateValueStrategy(UpdateValueStrategy.POLICY_NEVER));

		// add validation of required/supported
		final Object data = control.getData(DECORATION_KEY);
		if (data instanceof ControlDecoration) {
			bindings[2] = dbc.bindValue(JFaceObservables
					.observeControlDecoration((ControlDecoration) data),
					bindings[1].getValidationStatus(), null, null);
		}
		return bindings;
	}
}
