/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.exception;

import java.util.List;

import org.fortiss.tooling.kernel.extension.ITransformationProvider;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.data.TransformationProviderChain;

/**
 * An exception thrown by
 * {@link TransformationProviderChain#transform(Object, ITransformationContext)}
 * if the transformation failed at some point of the chain. The failed
 * transformation provider and the successful intermediate results can be
 * obtained from this exception.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: AB3D4D3B9B0CC64098BD7A9A03454214
 */
public class ChainTransformationFailedException extends
		TransformationFailedException {

	/** Stores a reference to the transformation provider chain. */
	private final TransformationProviderChain chain;

	/** Constructor. */
	public ChainTransformationFailedException(
			TransformationProviderChain chain,
			ITransformationProvider failedProvider,
			List<Object> successfulResults, TransformationFailedException cause) {
		super(failedProvider, successfulResults, cause);
		this.chain = chain;
	}

	/** Returns the original transformation chain. */
	public TransformationProviderChain getTransformationProviderChain() {
		return chain;
	}
}
