/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMLResource;

/**
 * Utility methods used for Eclipse resource storage.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 457FAB182DE3C79EDED1E33D6E744BC2
 */
public final class EMFResourceUtils {
	/**
	 * Stores the given EObject in the Eclipse workspace creating the given
	 * IFile.
	 */
	public static void createNewEclipseWorkspaceProject(IFile file,
			EObject topLevelElement) throws IOException {

		final Resource r = new ResourceSetImpl()
				.createResource(URI.createPlatformResourceURI(file
						.getFullPath().toString(), true));
		r.getContents().add(topLevelElement);
		r.save(buildOptionsMap());
	}

	/** Build the map of options used for IO of EMF models. */
	public static Map<String, Object> buildOptionsMap() {
		Map<String, Object> options = new HashMap<String, Object>();
		options.put(XMLResource.OPTION_ENCODING, "UTF-8");
		options.put(Resource.OPTION_ZIP, false);
		return options;
	}
}
