/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2005-2010 Technische Universitaet Muenchen                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IExtensionPoint;
import org.eclipse.core.runtime.Platform;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.osgi.framework.Bundle;

/**
 * Utility methods for the extension points of plug-ins.
 * 
 * Note: This class originates form the org.conqat.ide.commons.ui
 * implementation. It re-packaged with the kernel to avoid any dependency to
 * eclipse.ui packages. See http://www.conqat.org for the original version.
 * 
 * @author hummel
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: C3A56EFEB020008903564112B5DF8E59
 */
public class ExtensionPointUtils {

	/** Load the given class and throws an exception in case of problems. */
	public static Class<?> loadClass(String className, Bundle bundle)
			throws ClassNotFoundException {
		if (className == null) {
			throw new ClassNotFoundException("Class name not configured.");
		}
		return bundle.loadClass(className);
	}

	/** Returns the bundle an extension is declared in. */
	public static Bundle getBundle(IConfigurationElement configElement) {
		Bundle bundle = Platform.getBundle(configElement
				.getDeclaringExtension().getNamespaceIdentifier());
		return bundle;
	}

	/**
	 * Returns the {@link IConfigurationElement}s for a given extension point
	 * and element name.
	 */
	@SuppressWarnings("unchecked")
	public static List<IConfigurationElement> getConfigurationElements(
			String extensionPointName, String configurationElementName) {
		IExtensionPoint extensionPoint = Platform.getExtensionRegistry()
				.getExtensionPoint(extensionPointName);
		if (extensionPoint == null) {
			LoggingUtils.error(ToolingKernelActivator.getDefault(),
					"Could not find extension point " + extensionPointName);
			return Collections.EMPTY_LIST;
		}

		List<IConfigurationElement> configurationElements = new ArrayList<IConfigurationElement>();
		for (IExtension e : extensionPoint.getExtensions()) {
			for (IConfigurationElement ce : e.getConfigurationElements()) {
				if (ce.getName().equals(configurationElementName)) {
					configurationElements.add(ce);
				}
			}
		}
		return configurationElements;
	}
}